
# -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb - all rights reserved    
## No publication or distribution without authorization.

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

#Zope imports
from zope.interface import implements

# CMF imports
from Products.CMFCore.utils import SimpleItemWithProperties
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import permissions

#Archetypes imports
from Products.Archetypes.BaseFolder import BaseFolderMixin
from Products.Archetypes.public import *

#Product imports
from interfaces import IENSSkinnerControlPanel


#PROFILES : model for prefilled options


PROFILES = {'ensskinner' : {'model' : 'ensskinner',
                       'show_fixed_tabs' : False,
                       'show_breadcrumbs' : False,
                       'show_siteactions' : True,
                       'show_useractions' : True,
                       'personal_script_portlet' : (),
                       'left_portlets' :  ('here/portlet_navigation/macros/portlet'),
                       'right_portlets' : ('here/portlet_news/macros/portlet',
                                           'here/portlet_calendar/macros/portlet'),
                       },
            'full' :  {'model' : 'full',
                       'show_fixed_tabs' : True,
                       'show_breadcrumbs' : True,
                       'show_siteactions' : True,
                       'show_useractions' : True,
                       'personal_script_portlet' : (),
                       'left_portlets' : ('here/portlet_navigation/macros/portlet',
                                          'here/portlet_login/macros/portlet',
                                          'here/portlet_recent/macros/portlet',
                                          'here/portlet_related/macros/portlet'),
                       'right_portlets' : ('here/portlet_review/macros/portlet',
                                           'here/portlet_news/macros/portlet',
                                           'here/portlet_events/macros/portlet',
                                           'here/portlet_calendar/macros/portlet'),
                       },
            'simple': {'model' : 'simple',
                       'show_fixed_tabs' :  False,
                       'show_breadcrumbs' : False,
                       'show_siteactions' : False,
                       'show_useractions' : False,
                       'personal_script_portlet' : (),
                       'left_portlets' : (),
                       'right_portlets' : (),
                       },
            }

IMAGES = 'imgs'
ICONS = 'icons'
SELECTED_ICONS = 'selectedicons'
PORTLET_IMAGES = 'portletimages'


class ENSSkinnerControlPanel(BaseFolderMixin, UniqueObject):
    """
    ENSSkinner control panel tool : store/provide skin settings
    """
    isPrincipiaFolderish = True # Show up in the ZMI
    plone_tool = True
    portal_type = meta_type = "ENSSkinnerControlPanel"
    global_allow = False

    manage_options = BaseFolderMixin.manage_options
    __implements__ = BaseFolderMixin.__implements__

    security = ClassSecurityInfo()


    # profile (linked with default settings)
    # * ensskinner
    # * full
    # * simple
    model = "simple"
    
    # Activate/desactive
    show_fixed_tabs = None
    show_breadcrumbs = None
    show_siteactions = None
    show_useractions = None

    # Data portlet script links
    # Define new generic portlet considering Python script Id which provides
    # data to populate in. Ensure data provided by this script respect data
    # format to be WORKABLE.
    personal_script_portlet = []

    # Portlets dispatching (plone + personal)
    left_portlets = []
    right_portlets = []

    _properties = SimpleItemWithProperties._properties + (
        {'id':'model', 'type': 'string', 'mode':'w'},
        {'id':'show_fixed_tabs', 'type': 'boolean', 'mode':'w'},
        {'id':'show_breadcrumbs', 'type': 'boolean', 'mode':'w'},
        {'id':'show_siteactions', 'type': 'boolean', 'mode':'w'},
        {'id':'show_useractions', 'type': 'boolean', 'mode':'w'},
        {'id':'personal_script_portlet', 'type': 'lines', 'mode':'w'},
        {'id':'left_portlets', 'type': 'lines', 'mode':'w'},
        {'id':'right_portlets','type': 'lines', 'mode':'w'},
        )

    def __call__(self):
        """Redirect to ensskinner_config view"""
        self.REQUEST['RESPONSE'].redirect('ensskinner_config')

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        BaseFolderMixin.manage_afterAdd(self, item, container)
        self.invokeFactory(type_name='Folder', id=IMAGES)
        self.invokeFactory(type_name='Folder', id=ICONS)
        self.invokeFactory(type_name='Folder', id=SELECTED_ICONS)
        self.invokeFactory(type_name='Folder', id=PORTLET_IMAGES)
        self[IMAGES].setExcludeFromNav(True)
        self[ICONS].setExcludeFromNav(True)
        self[SELECTED_ICONS].setExcludeFromNav(True)
        self[PORTLET_IMAGES].setExcludeFromNav(True)
        self[PORTLET_IMAGES].setConstrainTypesMode(True)
        self[PORTLET_IMAGES].setLocallyAllowedTypes(['Image'])
        self[PORTLET_IMAGES].setImmediatelyAddableTypes(['Image'])

    def getPortletImagesFolder(self):
        """
        Return object image folder
        """
        return self[PORTLET_IMAGES]

    def getTopFolders(self):
        """
        Get level 1 folders (in aim to apply portugalisation)
        """
        ctool = getToolByName(self, 'portal_catalog')
        stool = getToolByName(self, 'portal_url')
        site = stool.getPortalObject()

        results = ctool(path={'query':'/'.join(site.getPhysicalPath()),
                              'depth': 1},)
        return results

    def applyModel(self, model):
        """
        When new model is applied, old customization are lost.
        """
        #No change for same model
        if model == self.model:
            return

        if model in PROFILES.keys():
            new_model = PROFILES[model]
            self.manage_changeProperties(new_model)
            self.updateSitePortlets()

    def updateNavLook(self, data):
        """
        Assign Icons & Images for each top folder ids given
        Some old images & icons can be cleaned.
        """
        for k in data.keys():
            type = k.split('_')[0]
            id = k.split('_')[1]

            if type == IMAGES:
                repository = self[IMAGES]
            if type == ICONS:
                repository = self[ICONS]
            if type == SELECTED_ICONS:
                repository = self[SELECTED_ICONS]

            if not id in repository.objectIds(): 
                repository.invokeFactory(type_name='Image',
                                         id=id)
            repository[id].setImage(data[k])

    def updateSitePortlets(self):
        """
        Update site properties (left_slots and right slots) using
        own properties (left_portlets and right_portlets)
        """
        ptool = getToolByName(self, 'portal_url')
        site = ptool.getPortalObject()
        site.left_slots = self.left_portlets
        site.right_slots = self.right_portlets
    
    def getNavIconURL(self, id, selected=False):
        """
        Return the navigation icon depending
        current position in site
        """
        if selected:
            type = SELECTED_ICONS
        else:
            type = ICONS
            
        if id in self[type].objectIds():
            return self[type][id].absolute_url()
        else:
            return None

    def getNavImgURL(self, id):
        """
        Return image to linked with 1 level folder
        """
        if id in self[IMAGES].objectIds():
            return self[IMAGES][id].absolute_url()
        else:
            return None

    def getPersonalPortlets(self):
        """
        Return list of (script, name) from 
        'personal_script_portlet' property
        """
        list  = []
        for script_id in self.personal_script_portlet:
            script = getattr(self, script_id, None)
            if script:
                values = ('portal/%s' % script_id, script_id)
                list.append(values)
        return list

    def getImagesPortletInfo(self):
        """
        Return list of dict (path, link URL)
        """
        list = []
        for img in self[PORTLET_IMAGES].objectValues(['ATImage',]):
            info = {'path' : img.absolute_url(), 'link': img.Description()}
            list.append(info)
        return list

    #Permissions functions

    def _isManager(self):
        """
        Return bool if logged user is Manager
        """
        mtool = getToolByName(self, 'portal_membership')
        user = mtool.getAuthenticatedMember()
        if user.has_role('Manager'):
            return True
        else:
            return False

    def displayFixedTabs(self):
        """
        Return bool case of property.
        Manager can bypass this one.
        """
        if self._isManager():
            return True

        if self.show_fixed_tabs:
            return True
        else:
            return False

    def displayBreadcrumbs(self):
        """
        Return bool case of property.
        Manager can bypass this one.
        """
        if self._isManager():
            return True

        if self.show_breadcrumbs:
            return True
        else:
            return False

    def displaySiteactions(self):
        """
        Return bool case of property.
        Manager can bypass this one.
        """
        if self._isManager():
            return True

        if self.show_siteactions:
            return True
        else:
            return False

    def displayUseractions(self):
        """
        Return bool case of property.
        Manager can bypass this one.
        """
        if self._isManager():
            return True

        if self.show_useractions:
            return True
        else:
            return False



InitializeClass(ENSSkinnerControlPanel)
