## -*- coding: utf-8 -*-
## Copyright (C) 2007 Ingeniweb - all rights reserved
## No publication or distribution without authorization.

from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.CMFPlone import PloneMessageFactory as _
from Products.Archetypes import atapi
from Products.Archetypes.Renderer import renderer
from Products.ENSSkinner.tool.controlpanel import PROFILES, \
                                                         IMAGES, \
                                                         ICONS, \
                                                         SELECTED_ICONS
from Products.ENSSkinner.config import I18N_DOMAIN, PORTLET_MAPPING_NAME

EditSchema = atapi.Schema((
    atapi.LinesField(
        'model',
        step=1,
        accessor='getModel',
        vocabulary='getModelVocabulary',
        widget=atapi.SelectionWidget(label='Model',
                                     label_msgid='model',
                                     i18n_domain=I18N_DOMAIN),
        ),
    atapi.LinesField(
        'show_fixed_tabs',
        step=2,
        accessor='hasFixedTabs',
        widget=atapi.BooleanWidget(label='Show fixed tabs',
                                   label_msgid='show_fixed_tabs',
                                   i18n_domain=I18N_DOMAIN),
        ),
    atapi.LinesField(
        'show_breadcrumbs',
        step=2,
        accessor='hasBreadcrumbs',
        widget=atapi.BooleanWidget(label='Show breadcrumbs',
                                   label_msgid='show_breadcrumbs',
                                   i18n_domain=I18N_DOMAIN),
        ),
    atapi.LinesField(
        'show_siteactions',
        step=2,
        accessor='hasSiteActions',
        widget=atapi.BooleanWidget(label='Show siteactions',
                                   label_msgid='show_siteactions',
                                   i18n_domain=I18N_DOMAIN),
        ),
    atapi.LinesField(
        'show_useractions',
        step=2,
        accessor='hasUserActions',
        widget=atapi.BooleanWidget(label='Show useractions',
                                   label_msgid='show_useractions',
                                   i18n_domain=I18N_DOMAIN,)
        ),
    atapi.LinesField(
        'personal_script_portlet',
        step=3,
        accessor='getPersonalScriptPortlet',
        widget=atapi.LinesWidget(label='Fill personal script portlet',
                                 label_msgid='script_portlet',
                     description="""Fill with Ids of your data scripts. This
                     portlet will be available in left and right portlet after
                     saving except if the given id does not exist.""",
                     description_msgid='script_portlet_tip',
                     i18n_domain=I18N_DOMAIN,),
        ),
   atapi.LinesField(
        'left_portlets',
        step=4,
        accessor='getLeftPortlets',
        vocabulary='getPortletVocabulary',
        widget=atapi.InAndOutWidget(label='Left portlets',
                                    label_msgid='left_portlets',
                                    i18n_domain=I18N_DOMAIN,),
        ),
   atapi.LinesField(
        'right_portlets',
        step=4,
        accessor='getRightPortlets',
        vocabulary='getPortletVocabulary',
        widget=atapi.InAndOutWidget(label='Right portlets',
                                    label_msgid='right_portlets',
                                    i18n_domain=I18N_DOMAIN,),
        ),
    ),
)

class ENSSkinnerControlPanelView(BrowserView):
    """
    ENSSkinner control panel view
    """

    ensskinner_tool = None

    def __init__(self, context, request):
        BrowserView.__init__(self, context, request)
        self.context = context
        self.ensskinner_tool = getToolByName(context, 'ensskinner_controlpanel')
        self.schema = EditSchema.copy() + self.generateNavigationSchema().copy()
#        self.schema = EditSchema.copy()

    def getEditFields(self, step):
        fields = [f for f in self.schema.copy().fields() if f.step == step]
        for f in fields:
            if f.vocabulary:
                method = getattr(self, f.vocabulary)
                f.vocabulary = method()
        return fields

    def renderWidget(self, instance, field, mode):
        """Render empty widget from field"""
        field_name = field.getName()

        # GRUIK touch'
        # this test bypass accessor with ImageField
        # Go on workable, but not really good for ergonomic
        if field_name.startswith(IMAGES) or \
           field_name.startswith(ICONS) or \
           field_name.startswith(SELECTED_ICONS):
            field.widget.macro =  "fixed_img_widget"
            accessor = field.getAccessor(instance)
        else:
            accessor = getattr(self, field.accessor)

        widget = field.widget
        return renderer.render(field_name=field_name,
                               mode=mode,
                               widget=widget,
                               instance=instance,
                               field=field,
                               accessor=accessor)


    def getModel(self):
        return self.ensskinner_tool.model

    def getModelVocabulary(self):
        list = [(k, k) for k in PROFILES.keys()]
        return atapi.DisplayList(list)

    def edit(self):
        """
        Process form data
        """
        request = self.context.request
        form = request.form
        data = {}

        #format data to update tool properties

        #String model
        #STOP update process if new model is set 
        if self.ensskinner_tool.model != form['model']:
            self.ensskinner_tool.applyModel(form['model'])
        else:
            #Boolean (show/hide)
            checked_keys = [k for k in form.keys() if k.startswith('show') and form[k] == '1']
            unchecked_keys = [k for k in form.keys() if k.startswith('show') and form[k] == '0']
            for k in checked_keys:
                data.update({k:True})
            for k in unchecked_keys:
                data.update({k:False})

            #Create images
            images = {}
            [images.update({k:form[k]}) for k in form.keys() if k.endswith('file')]
            self.ensskinner_tool.updateNavLook(images)

            #Personal scripts portlet
            data.update({'personal_script_portlet':form['personal_script_portlet']})

            #Portlets
            data.update({'left_portlets':form['left_portlets']})
            data.update({'right_portlets':form['right_portlets']})

            #apply changes
            self.ensskinner_tool.manage_changeProperties(data)
            self.ensskinner_tool.updateSitePortlets()

        msg = _(u'Changes saved.')
        self.context.plone_utils.addPortalMessage(msg)
        request.RESPONSE.redirect('ensskinner_config')


    def hasFixedTabs(self):
        """
        Get tool property
        """
        return self.ensskinner_tool.show_fixed_tabs

    def hasBreadcrumbs(self):
        """
        Get tool property
        """
        return self.ensskinner_tool.show_breadcrumbs

    def hasSiteActions(self):
        """
        Get tool property
        """
        return self.ensskinner_tool.show_siteactions

    def hasUserActions(self):
        """
        Get tool property
        """
        return self.ensskinner_tool.show_useractions

    def getPersonalScriptPortlet(self):
        """
        Get tool property
        """
        return self.ensskinner_tool.personal_script_portlet

    def generateNavigationSchema(self):
        """
        Complete the edit schema with image fields
        (Depends on site structure)
        """
        folders = self.ensskinner_tool.getTopFolders()
        schema = atapi.Schema()
        for f in folders:
            img_id = '%s_%s' % (IMAGES, f['getId'])
            img_default_label = 'image for %s' % f['getId']
            icon_id = '%s_%s' % (ICONS, f['getId'])
            icon_default_label = 'icon for %s' % f['getId']
            icon_selected_id = '%s_%s' % (SELECTED_ICONS, f['getId'])
            icon_selected_default_label = 'icon selected for %s' % f['getId']
            field = atapi.ImageField(img_id,
                                     step=5,
                                     widget=atapi.ImageWidget(label=img_default_label,
                                                              i18n_domain=I18N_DOMAIN,),
                                     )
            schema.addField(field)
            field = atapi.ImageField(icon_id,
                                     step=5,
                                     widget=atapi.ImageWidget(label=icon_default_label,
                                                              i18n_domain=I18N_DOMAIN,),
                                     )
            schema.addField(field)
            field = atapi.ImageField(icon_selected_id,
                                     step=5,
                                     widget=atapi.ImageWidget(label=icon_selected_default_label,
                                                              i18n_domain=I18N_DOMAIN,),
                                     )
            schema.addField(field)
        return schema

    def getPortletVocabulary(self):
        """
        Provides basic plone portlets and personal portlets
        """
        portlets = []
        portlets.extend(self.ensskinner_tool.getPersonalPortlets())
        portlets.extend(PORTLET_MAPPING_NAME)
        return portlets

    def getLeftPortlets(self):
        """
        """
        return self.ensskinner_tool.left_portlets

    def getRightPortlets(self):
        """
        """
        return self.ensskinner_tool.right_portlets

